<?php

    class In8_Customer {

        private $user_id = 0;
        public static $customerFieldMapping = [
            'customform' => 0,
            'paymentmethod_AmericanExpress' => 0,
            'paymentmethod_Discover' => 0,
            'paymentmethod_Visa' => 0,
            'paymentmethod_MasterCard' => 0,
            'paymentmethod_Paypal' => 0,
            'currency_EUR' => 0,
            'currency_USD' => 0,
            'currency_GBP' => 0,
            'taxable' => "T",
            'subsidiary' => 0,
            'category' => 0
        ];
        private static $customerFieldMappingHelpText = [
            'customform' => "Customer customform internal ID",
            'paymentmethod_AmericanExpress' => "NetSuite internal ID for American Express payment method",
            'paymentmethod_Discover' => "NetSuite internal ID for Discover payment method",
            'paymentmethod_Visa' => "NetSuite Internal ID for Visa payment method",
            'paymentmethod_MasterCard' => "NetSuite Internal ID for Mastercard payment method",
            'paymentmethod_Paypal' => "NetSuite internal ID for Paypal payment method",
            'currency_EUR' => "NetSuite currency internal ID for EUR",
            'currency_USD' => "NetSuite currency internal ID for USD",
            'currency_GBP' => "NetSuite currency internal ID for GBP",
            'taxable' => "T/F, if T this customer pays sales tax. if F this customer is not taxable",
            'subsidiary' => "Customer subsidiary internal ID",
            'category' => "Customer category internal ID"
        ];
        //This array contains all required conditional mappings
        private static $prefinedCondMapping = [
//        [
//            'type' => 1,
//            'operator' => 3,
//            'ns_field_key' => 'entityid',
//            'wc_field_key' => 'email',
//            'wc_field_value_prefix' => '',
//            'required' => 1
//        ]
        ];
        //These are the currently supported payment gateways
        public static $allowedGatways = [
            'cybersource' => 'Cyber Source',
            'paypal_pro' => 'Paypal Pro',
            'mes_cc' => 'MES Credit Card',
            'ajdg_merchant_esolutions' => 'Merchant E-Solutions',
            'WorldPayCw_Visa' => 'World Pay VISA Plugin',
            'WorldPayCw_CreditCard' => 'World Pay Credit Card Plugin',
            'paypal_pro_payflow' => 'Paypal pro payflow',
            'sagepaydirect' => 'SagePay Direct',
            'braintree' => 'Braintree Gateway',
            'in8_netsuite_payments' => 'IN8 NetSuite Payments',
            'cybsawm' => 'Cybersource',
            'ajdg_merchant_esolutions' => 'Merchant E-Solutions',
            'first_data_payeezy_credit_card' => 'First Data Payeezy Credit Card',
			'cybersource_credit_card' => 'Cyber Source',
        ];
        //plugin tag name
        public static $optionsTag = 'In8_Customer';
        public static $optionsTagCM = 'In8_Customer_CM'; //conditional mapping
        public static $optionsTagCO = 'In8_Customer_CO'; //configurable options
        public static $optionsTagFP = 'In8_Customer_FP'; //forms and payment methods

	    private static $instance = NULL;
	    static public function getInstance() {
		    if ( self::$instance === NULL ) {
			    self::$instance = new In8_Customer();
		    }
		
		    return self::$instance;
	    }

        public function __construct() {
	    	
	        add_action( 'in8sync_async_addUpdateNetsuiteCustomer', [ $this, 'addUpdateNetsuiteCustomer' ], 10, 2 );

            //tab link and content
            add_filter('in8_admin_tab', [$this, 'addTab']);
            add_action('in8_netsuite_top_level_tabs_page', [$this, 'tabContent']);

            //ajax for saving plugin admin settings
            add_action('wp_ajax_' . self::getOptionsTag(), [$this, 'saveForm']);

            //ajax for saving plugin admin settings
            add_action('wp_ajax_' . self::$optionsTagCO, [$this, 'saveConfigurationalOptionsForm']);

            //ajax for getting saved forms and payment methods
            add_action('wp_ajax_' . self::$optionsTagFP, [$this, 'getFormsAndPaymentMethodsFromNS']);

            $co_options = self::getOptions(2);

            if (isset($co_options['enable_customer_sync']) && $co_options['enable_customer_sync'] == 'on') {
                //USER PROFILE HOOKS
				if ( apply_filters( 'in8_sync_customer_on_register', TRUE ) ) {
		            //wordpress user register
					add_action( 'user_register', [ $this, 'handleAsyncCustomer' ], 10000 );
		            //wooocommerce customer created
					add_action( 'woocommerce_created_customer', [ $this, 'handleAsyncCustomer' ], 10000 );
	            }
	            
	            // enable gravity form registration to sync to netsuite
	            if( apply_filters('in8_customer_enable_gform_registration', false) ){
		            add_action('gform_user_registered', [$this, 'handleAsyncCustomer']);
	            }
				//Gravity Forms submit
				add_action( 'gform_after_submission', function($entry){
        
				    // adding the gravity form id when sending to ns
                    add_filter('in8_after_customer_data_mapping', function($customer_data)use($entry){
                        $customer_data['custentity_gravity_id_comp'] = $entry['form_id'];
	                    return $customer_data;
                    });
				    
					$this->addUpdateNetsuiteCustomer($entry['created_by']);
				}, 10, 2);
				
	
				// check if should we run the sync on the wp-admin user profile
                if( isset($co_options['enable_wp_profile_change_sync']) && $co_options['enable_wp_profile_change_sync'] == 'on' && is_admin() ){
					//hook for detecting update in user profile
					add_action('profile_update', array($this, 'handleAsyncCustomer'));
				}
	
				// check if should we run the sync on the front my account
				if( isset($co_options['enable_profile_change_sync']) && $co_options['enable_profile_change_sync'] == 'on' ) {
					//hook for detecting customer address save
					add_action('woocommerce_customer_save_address', [$this, 'handleAsyncCustomer']);
					add_action('profile_update', array($this, 'handleAsyncCustomer'));
				}else{
					// sync of profile changes on my account are disabled
					// remove the profile update only on the front side
					if( !is_admin() ){
						remove_action('profile_update', array($this, 'handleAsyncCustomer'));
					}
				}
	
				//load endpoints
                add_action('woocommerce_api_loaded', array($this, 'loadEndpoints'));
	
	            // MULTIPLE addresses
	            add_action('wcmca_after_adding_new_user_address', [$this, 'create_new_address'], 10, 2);
//				add_action('wcmca_after_updating_user_address', [$this, 'update_address'], 10, 3);
	            add_action('wp_enqueue_scripts', function(){
		            if( !is_checkout() ) return;
		            wp_enqueue_script('multiple-addresses-fix', In8_Customers_URL.'inc/assets/js/multiple-addresses.js', ['jquery'], rand(0,10000));
	            });
	            add_action('woocommerce_checkout_update_order_meta', [$this, 'save_form_selected_address']);
            }

        }
	
	
	
	    public function handleAsyncCustomer( $customer_id ) {
		    $customer_plugin_conf_options = In8_Customer::getOptions( 2 );
		    if (
			    ! empty( $customer_plugin_conf_options ) &&
			
			    isset( $customer_plugin_conf_options['use_async'] ) &&
			    $customer_plugin_conf_options['use_async'] == 'on'
		    ) {
			    wp_schedule_single_event( time() + 15, 'in8sync_async_addUpdateNetsuiteCustomer', [ $customer_id, true ] );
		    } else {
			    $this->addUpdateNetsuiteCustomer( $customer_id );
		    }
	    }
        

        public function addTab($tabs) {
            $tabs[self::$optionsTag] = ['label' => "Customers", "priority" => 3];
            return $tabs;
        }

        public function tabContent() {
            $options = self::options();

            //this handles logic for putting empty mapping keys
            $default_keys = array_keys(self::$customerFieldMapping);
            $default_values = array_values(self::$customerFieldMapping);

            if (empty($options['mapping_key']) || empty($options['mapping_value'])) {
                $options['mapping_key'] = $default_keys;
                $options['mapping_value'] = $default_values;
            } else {
                foreach ($default_keys as $default_key) {
                    if (!in_array($default_key, $options['mapping_key'])) {
                        $options['mapping_key'][] = $default_key;
                        $options['mapping_value'][] = 0;
                    }
                }
            }


            $plugin_gateways = self::_getNetSuitePaymentMethods();
            $active_payment_gateways = self::getActivePaymentGateways();
            //conditional mapping options
            $cm_options = self::getOptions(1);
            $required_key_present = [];
            if (!empty($cm_options)) {
                foreach ($cm_options as $key => $mapping) {
                    $index = $key + 1;

                    if (!empty($mapping['operator']) && ($mapping['operator'] == 2 || !empty($mapping['type']))) {

//                        foreach (self::$prefinedCondMapping as $pre_mapping_key => $pre_mapping) {
//                            if ($mapping['operator'] == $pre_mapping['operator'] && $mapping['type'] == $pre_mapping['type'] && $mapping['ns_field_key'] == $pre_mapping['ns_field_key']) { //check for required mapping
//                                $required_key_present[] = $pre_mapping_key;
//                                $cm_options[$key]['required'] = 1;
//                            }
//                        }
                        if ($mapping['operator'] == 2) {
                            $mapping['type'] = 0;
                        }
                        $cm_options[$key]['template'] = ConditionalMapping::getCMTemplate($mapping['type'], $mapping['operator'], $index, $mapping);
                    } else {
                        unset($cm_options[$key]);
                    }
                }
            }


            foreach (self::$prefinedCondMapping as $pre_mapping_key => $pre_mapping) {
                $cm_count = count($cm_options);

                if (!in_array($pre_mapping_key, $required_key_present)) { //insert required key if not present
                    $cm_options[$cm_count] = $pre_mapping;
                    $cm_options[$cm_count]['template'] = ConditionalMapping::getCMTemplate($pre_mapping['type'], $pre_mapping['operator'], $cm_count, $pre_mapping);
                }
            }

            $roles = get_editable_roles();
            //configuration mapping options
            $co_options = self::getOptions(2);

            $helptext = self::$customerFieldMappingHelpText;
            //if membership enabled get all defined plans
            if (defined('MEMBERSHIP_PLUGIN_ENABLED')) {
                $membership_plans = wc_memberships_get_membership_plans();
            }

            $forms_and_payments = In8_Customer::getOptions(3);
            if (isset($forms_and_payments['customer_forms']) && !empty($forms_and_payments['customer_forms'])) {
                $customer_forms = $forms_and_payments['customer_forms'];
            }
            if (isset($forms_and_payments['payment_methods']) && !empty($forms_and_payments['payment_methods'])) {
                $payment_methods = $forms_and_payments['payment_methods'];
            }

            require_once('admin-setting-tab-customers.php');
        }

        //load endpoints
        public function loadEndpoints() {
            require_once plugin_dir_path(__FILE__) . 'api_classes/class_wc_api_meta_customer.php';
            require_once plugin_dir_path(__FILE__) . 'api_classes/class_wc_api_getroles.php';
            require_once plugin_dir_path(__FILE__) . 'api_classes/class_wc_api_meta_post.php';
            require_once plugin_dir_path(__FILE__) . 'api_classes/class_wc_api_meta_get.php';
            require_once plugin_dir_path(__FILE__) . 'api_classes/class_wc_api_meta_search.php';
            require_once plugin_dir_path(__FILE__) . 'api_classes/class_wc_api_anu.php';

            // filter the array of default api classes
            add_filter('woocommerce_api_classes', array($this, 'register'));
        }

        //register endpoints
        public function register($api_classes = array()) {
            array_push($api_classes, 'meta_customer');
            array_push($api_classes, 'getroles');
            array_push($api_classes, 'meta_post');
            array_push($api_classes, 'meta_get');
            array_push($api_classes, 'meta_search');
            array_push($api_classes, 'WC_API_Anu');
            return $api_classes;
        }
	
	    private $customer_internal_id = 0, $contact_id = 0;

        public function addUpdateNetsuiteCustomer($customer_id, $reset = false) {
            $active_plugins = apply_filters('active_plugins', get_option('active_plugins'));

            //check if "send all order as purchase order" option is enabled, then send a customer as vendor
            if (in_array('in8sync-purchaseorder/in8sync-purchaseorder.php', $active_plugins)) {
                $plugin_conf_options = In8_PurchaseOrder::getOptions(2);

                if (isset($plugin_conf_options['all_purchase_orders'])) {
                    return FALSE; //return if admin only want to use purchase order, vendor sync will sync the user to NS
                }
            }

            //this will check if this is called from checkout page and sales order or purchase order extension is installed
            if (is_page(wc_get_page_id('checkout')) && (in_array('in8sync-salesorder/in8sync-salesorder.php', $active_plugins) || in_array('in8sync-purchaseorder/in8sync-purchaseorder.php', $active_plugins))) {
                return FALSE; //disable on checkout page
            }

            //disable when hook executed from api
			if ( strpos( $_SERVER['REQUEST_URI'], 'wc-api' ) !== FALSE && ! preg_match( '/dynamics_api/i', $_SERVER['REQUEST_URI'] ) ) {
                return FALSE; //disable on API request
            }
            $this->user_id = $customer_id;
            $this->sendToNS();
        }

        public function sendToNS() {
	        if( apply_filters('in8_customer_doNotSync', false, $this->user_id) ) return;
        	
            //get user data
            $woo_customer_data = get_userdata($this->user_id);

            //configuration mapping options
            $plugin_conf_options = self::getOptions(2);

            //check for alloed roles . Also customer role allowed by default
			if( !empty($woo_customer_data->roles[0]) ){
				$first_role_key = 0;
			}else {
				reset( $woo_customer_data->roles );
				$first_role_key = key( $woo_customer_data->roles );
			}
			if ($woo_customer_data->roles[$first_role_key] == 'customer' ||
                    (isset($plugin_conf_options['allowed_roles']) &&
                    is_array($plugin_conf_options['allowed_roles']) &&
			     in_array($woo_customer_data->roles[$first_role_key], $plugin_conf_options['allowed_roles']))) {

                //netsuite API connection
                $netsuite_connection = new NetsuiteConnection($this->user_id, 'customer');

                //customer internal ID
                $customer_internal_id = get_user_meta($this->user_id, 'customerinternalid', true);
                $contact_internal_id = get_user_meta($this->user_id, 'contactinternalid', true);

                $is_update = FALSE;
                //GET ALL USER META DATA
                $meta_fields = get_user_meta($this->user_id);
				$parentc = (isset($meta_fields['parentcompany'][0])) ? $meta_fields['parentcompany'][0] : '';
				$parentuser = get_users(array('meta_key' => 'customerinternalid', 'meta_value' => $parentc, 'number' => 1, 'fields' => 'ids', 'count_total' => false));
					foreach($parentuser as $puser){
						$parentuserid = $puser;
					}								
				$parenttax = get_user_meta($parentuserid, 'taxable', true);

				if(empty($parentc)){
					$phone = (isset($meta_fields['billing_phone'][0])) ? $meta_fields['billing_phone'][0] : '';
					$company = (isset($meta_fields['billing_company'][0])) ? $meta_fields['billing_company'][0] : '';
					$zipcode = (isset($meta_fields['billing_postcode'][0])) ? $meta_fields['billing_postcode'][0] : '';
				}
	
	            $contactSearchIsOn = !empty($meta_fields['billing_company'][0]) && isset($plugin_conf_options['enable_contact_sync']) && $plugin_conf_options['enable_contact_sync'] == 'on';
	            if (!empty($customer_internal_id)) {
                    $this->customer_internal_id = $customer_internal_id; //customer update
	                if( $contactSearchIsOn && !empty($contact_internal_id) ){
	                	$this->contact_id = $contact_internal_id;
	                }
                    $is_update = TRUE;
                } else {
					$include_company = isset($plugin_conf_options['search_include_company']) ? $plugin_conf_options['search_include_company'] : '';
					$include_phone = isset($plugin_conf_options['search_include_phone']) ? $plugin_conf_options['search_include_phone'] : '';
	
	                // SEARCH 1 - search for contact if contacts are enabled or search for customer if contact is not enabled, by email and company name(if company name is enabled)
	
	                $search_by = 'customer';
	                if( $contactSearchIsOn ){
		                $search_by = 'contact';
	                }
	                $netsuite_search_connection = new NetsuiteConnection( $this->order_id, 'salesorder-'.$search_by.'-search' );
	
	                //get filter expression for search
	                $filterExpression = CustomerHelperFunctions::getCustomerFilterExpression( $woo_customer_data->data->user_email, $include_company, $company, $include_phone, $phone, $plugin_conf_options );
	                //customer search request
	                $customer_search = $netsuite_search_connection->doRequest( [], [ 'filterExpression' => $filterExpression ] );
	                //if search returns result
	                if ( $customer_search['success'] == 1 && ! empty( $customer_search['data'] ) ) {
		                // if it returns 1 or more results we need to make sure they follow the other filters(email, company, phone) if any of them are enabled
		                if ( count( $customer_search['data'] ) >= 1 ) {
			                //filter search to get internal id
			                if( $contactSearchIsOn ) {
				                $item_result = CustomerHelperFunctions::processContactSearchResult( $customer_search['data'], $woo_customer_data->data->user_email, $include_company, $company, $include_phone, $phone, FALSE );
			                }else{
				                $item_result = CustomerHelperFunctions::processCustomerSearchResult( $customer_search['data'], $woo_customer_data->data->user_email, $plugin_conf_options['search_include_company'], $company, $plugin_conf_options['search_include_phone'], $phone, false );
			                }
			                // Checks if the return is a contact information
			                if ( $item_result['isContact'] == TRUE ) {
				                $this->customer_internal_id = $item_result['company']['internalId'];
				                $this->contact_id           = $item_result['internalId'];
			                } else {
				                $this->customer_internal_id = $item_result['internalId'];  //customer update on email search
			                }
		                }
	                }
	
	                //Search by ending of email address
	                $search_ending_of_email = isset($plugin_conf_options['search_email_domain']) && $plugin_conf_options['search_email_domain'] == 'on';
	                if( empty($this->customer_internal_id) && $contactSearchIsOn && $search_ending_of_email ){
		                $netsuite_search_connection = new NetsuiteConnection( $this->order_id, 'salesorder-customer-search' );
		
		                //get filter expression for search
		                $filterExpression = CustomerHelperFunctions::getEmailEndingFilterExpression( $woo_customer_data->data->user_email, $plugin_conf_options );
		
		                //customer search request
		                $customer_search = $netsuite_search_connection->doRequest( [], [ 'filterExpression' => $filterExpression ] );
		                if ( $customer_search['success'] == 1 && ! empty( $customer_search['data'] ) ) {
			                //if email search returns more than one result
			                if ( count( $customer_search['data'] ) >= 1 ) {
				                //filter search to get internal id
				                $item_result = CustomerHelperFunctions::processCustomerSearchResult( $customer_search['data'], $woo_customer_data->data->user_email, $include_company, $company, $include_phone, $phone, FALSE );
				                // Checks if the return is a contact information
				                if ( $item_result['isContact'] == TRUE ) {
					                $this->customer_internal_id = $item_result['company']['internalId'];
					                $this->contact_id           = $item_result['internalId'];
				                } else {
					                $this->customer_internal_id = $item_result['internalId'];  //customer update on email search
				                }
			                }
		                }
	                }
	
	                // contact was not found, search for customer
	                if( empty($this->customer_internal_id) && $contactSearchIsOn ){
		
		                $search_by = 'customer';
		                $netsuite_search_connection = new NetsuiteConnection( $this->order_id, 'salesorder-'.$search_by.'-search' );
		
		                //get filter expression for search
		                $filterExpression = CustomerHelperFunctions::getCustomerFilterExpression( $woo_customer_data->data->user_email, $include_company, $company, $include_phone, $phone );
		                //customer search request
		                $customer_search = $netsuite_search_connection->doRequest( [], [ 'filterExpression' => $filterExpression ] );
		                //if serach returns result
		                if ( $customer_search['success'] == 1 && ! empty( $customer_search['data'] ) ) {
			                // if it returns 1 or more results we need to make sure they follow the other filters(email, company, phone) if any of them are enabled
			                if ( count( $customer_search['data'] ) >= 1 ) {
				                //filter search to get internal id
				                $item_result = CustomerHelperFunctions::processCustomerSearchResult( $customer_search['data'], $woo_customer_data->data->user_email, $include_company, $company, $include_phone, $phone, false );
				                // Checks if the return is a contact information
				                if ( $item_result['isContact'] == TRUE ) {
					                $this->customer_internal_id = $item_result['company']['internalId'];
					                $this->contact_id           = $item_result['internalId'];
				                } else {
					                $this->customer_internal_id = $item_result['internalId'];  //customer update on email search
				                }
			                }
			
		                }
	                }
		
                }

                //plain mapping
                $plugin_options = self::getOptions();

                $plugin_settings_map = $plugin_settings_update_map = [];

                if (!empty($plugin_options['mapping_key']) && !empty($plugin_options['mapping_value'])) {
                    $plugin_settings_map = array_combine($plugin_options['mapping_key'], $plugin_options['mapping_value']);
                    $plugin_settings_map = array_change_key_case($plugin_settings_map);
                    if ($is_update && !empty($plugin_options['mapping_update'])) {
                        $plugin_settings_update_map = array_combine(array_intersect_key($plugin_options['mapping_key'], $plugin_options['mapping_update']), array_intersect_key($plugin_options['mapping_update'], $plugin_options['mapping_key']));
                        $plugin_settings_update_map = array_change_key_case($plugin_settings_update_map);
                    }
                }



                //CURRENCY
                $default_currency = strtolower(get_woocommerce_currency());

                if (!$is_update || ($is_update && !isset($plugin_settings_update_map['currency_' . $default_currency]))) {
                    if (isset($plugin_conf_options['currency']) && $plugin_conf_options['currency'] == 'on') {
                        //check if currency needed
                        if (empty($plugin_settings_map['currency_' . $default_currency])) {
                            $error = "Customer data can't be sent to netsuite as order currency(" . $default_currency . ") isn't mapped to netsuite.";
                            $netsuite_connection->processError($error);
                            return FALSE; //halt the script
                        } else {
                            //currency array
                            $customer_data['currency'] = ["currency" => $plugin_settings_map['currency_' . $default_currency]];
                        }
                    }
                }

                //SUBSIDIARY
                if (!$is_update || ($is_update && !isset($plugin_settings_update_map['subsidiary']))) {
                    if (!empty($plugin_settings_map['subsidiary'])) {
                        $customer_data['subsidiary'] = $plugin_settings_map['subsidiary'];
                    }
                }

                //CUSTOM FORM
                if (!$is_update || ($is_update && !isset($plugin_settings_update_map['customform']))) {
                    $customer_data['customform'] = $plugin_settings_map['customform'];
                }
                //CONTACT FORM
                if (!$is_update || ($is_update && !isset($plugin_settings_update_map['contact.customform']))) {
                    $customer_data['contact.customform'] = $plugin_settings_map['contact.customform'];
                }

                //TAXABLE
				if($parentc != ''){
					$customer_data['taxable'] = $parenttax;
				 }else{
					 if (!$is_update || ($is_update && !isset($plugin_settings_update_map['taxable']))) {
						  if (array_key_exists('taxable', $plugin_settings_map) && !empty($plugin_settings_map['taxable'])) {
							  $customer_data['taxable'] = $plugin_settings_map['taxable'];
						  }
					 }
				 }

                //CATEGORY
                if (!$is_update || ($is_update && !isset($plugin_settings_update_map['category']))) {
                    if (($is_update && !isset($plugin_settings_update_map['category'])) && array_key_exists('category', $plugin_settings_map) && !empty($plugin_settings_map['category'])) {
                        $customer_data['category'] = $plugin_settings_map['category'];
                    }
                }


                $customer_data['newaddress'] = 'T';                
				if ( isset( $plugin_options['new_address'] ) && $plugin_options['new_address'] == 'on' ) {
					$customer_data['newaddress'] = "T";
				} else {
					$customer_data['newaddress'] = "F";
				}
				
				
                $customer_data['recordType'] = "customer";
                $customer_data['referrer'] = "http://" . $_SERVER['SERVER_NAME'];
				if(empty($parentc)){
					$customer_data['custentity_in8_wc_customer_id'] = $this->user_id;
					$customer_data['email'] = $woo_customer_data->data->user_email;
				}

                if (!empty($woo_customer_data->first_name) && !empty($woo_customer_data->last_name)) {
                    $customer_data['firstname'] = $woo_customer_data->first_name;
                    $customer_data['lastname'] = $woo_customer_data->last_name;
                } else {
                    $customer_data['firstname'] = (isset($meta_fields['billing_first_name'][0])) ? $meta_fields['billing_first_name'][0] : '';
                    $customer_data['lastname'] = (isset($meta_fields['billing_last_name'][0])) ? $meta_fields['billing_last_name'][0] : '';
                }

                $customer_data['companyname'] = $company;
                $customer_data['phone'] = $phone;

                $customer_data['billaddr1'] = (isset($meta_fields['billing_address_1'][0])) ? $meta_fields['billing_address_1'][0] : '';
                $customer_data['billaddr2'] = (isset($meta_fields['billing_address_2'][0])) ? $meta_fields['billing_address_2'][0] : '';
                $customer_data['billcity'] = (isset($meta_fields['billing_city'][0])) ? $meta_fields['billing_city'][0] : '';
                $customer_data['billstate'] = (isset($meta_fields['billing_state'][0])) ? $meta_fields['billing_state'][0] : '';
                $customer_data['billzip'] = (isset($meta_fields['billing_postcode'][0])) ? $meta_fields['billing_postcode'][0] : '';
                $customer_data['billcountry'] = (isset($meta_fields['billing_country'][0])) ? $meta_fields['billing_country'][0] : '';
                $customer_data['billphone'] = $phone;

                $customer_data['shipaddr1'] = (isset($meta_fields['shipping_address_1'][0])) ? $meta_fields['shipping_address_1'][0] : '';
                $customer_data['shipaddr2'] = (isset($meta_fields['shipping_address_2'][0])) ? $meta_fields['shipping_address_2'][0] : '';
                $customer_data['shipcity'] = (isset($meta_fields['shipping_city'][0])) ? $meta_fields['shipping_city'][0] : '';
                $customer_data['shipstate'] = (isset($meta_fields['shipping_state'][0])) ? $meta_fields['shipping_state'][0] : '';
                $customer_data['shipzip'] = (isset($meta_fields['shipping_postcode'][0])) ? $meta_fields['shipping_postcode'][0] : '';
                $customer_data['shipcountry'] = (isset($meta_fields['shipping_country'][0])) ? $meta_fields['shipping_country'][0] : '';
	            if( !empty($customer_data['shipcountry']) ) {
		            $customer_data['shipphone'] = isset( $meta_fields['shipping_country'][0] ) && ! empty( $phone ) ? $phone : '';
	            }

                $bill_fname = (isset($meta_fields['billing_first_name'][0])) ? $meta_fields['billing_first_name'][0] : '';
                $bill_lname = (isset($meta_fields['billing_last_name'][0])) ? $meta_fields['billing_last_name'][0] : '';

                 //IS PERSON
				if(empty($parentc)){
					if (!empty($company) || isset($plugin_conf_options['disable_is_person']) && $plugin_conf_options['disable_is_person'] == 'on') {
						$customer_data['isperson'] = "F";
					} else {
						$customer_data['isperson'] = "T";
					}
				}

                 //bill addressee
				if(empty($parentc)){
					if (!empty($bill_fname) || !empty($bill_lname)) {
						$customer_data['billaddressee'] = trim($bill_fname . ' ' . $bill_lname);
						if (!empty($company)) {
							$customer_data['billaddressee'] = $company;
							$customer_data['billattention'] = trim($bill_fname . ' ' . $bill_lname);
						}
					} elseif (!empty($company)) {
						$customer_data['billaddressee'] = $company;
						$customer_data['billattention'] = trim($bill_fname . ' ' . $bill_lname);
					}
				}

                //ship addressee
				if(empty($parentc)){
					$ship_fname = (isset($meta_fields['shipping_first_name'][0])) ? $meta_fields['shipping_first_name'][0] : '';
					$ship_lname = (isset($meta_fields['shipping_last_name'][0])) ? $meta_fields['shipping_last_name'][0] : '';
					$ship_comp = (isset($meta_fields['shipping_company'][0])) ? $meta_fields['shipping_company'][0] : '';

					if (!empty($ship_fname) || !empty($ship_lname)) {
						$customer_data['shipaddressee'] = trim($ship_fname . ' ' . $ship_lname);
						if (!empty($ship_comp)) {
							$customer_data['shipaddressee'] = $ship_comp;
							$customer_data['shipattention'] = trim($ship_fname . ' ' . $ship_lname);
						}
					} elseif (!empty($ship_comp)) {
						$customer_data['shipaddressee'] = $ship_comp;
						$customer_data['shipattention'] = trim($ship_fname . ' ' . $ship_lname);
					}
				}

                //remove empty values
                $customer_data = array_filter($customer_data);
                //apply conditional mapping
                $this->conditionalMapping($customer_data, NULL, $is_update);
	
	            $customer_data = apply_filters('in8_after_customer_data_mapping', $customer_data, $this->user_id);

				$customer_data['source'] = "https://" . $_SERVER['HTTP_HOST'];


	            if( (empty($customer_data['firstname']) && empty($customer_data['lastname']) && empty($customer_data['companyname'])) ){
	            	$netsuite_connection->processError("The customer is missing firstname, lastname and companyname");
	            }
				
                // It sends to NetSuite only if there is data on the first name, last name or company. Otherwise NetSuite returns error
                if (!empty($customer_data) && (!empty($customer_data['firstname']) || !empty($customer_data['lastname']) || !empty($customer_data['companyname']))) {
                    //customer create/update request
	                if( !empty($this->customer_internal_id) ){
	                	if( $contactSearchIsOn ){
			                if( !empty($this->contact_id) ){
				                // just update the contact
				                $customer_data['recordType'] = 'contact';
				                $customer_data['internalId'] = $this->contact_id;
				                $customer_data['company'] = $this->customer_internal_id;
			                }else{
			                	// just create a new contact
				                $customer_data['recordType'] = 'contact';
				                $customer_data['company'] = $this->customer_internal_id;
			                }
		                }else{
							// update the customer
			                $customer_data['internalId'] = $this->customer_internal_id;
		                }
	                }else{
	                	// create new customer | nothing needs to happen here
	                }
	                
	                $company_data = false;
	                if( $customer_data['recordType'] == 'contact' ){
	                	$company_data = $customer_data;// Keep a backup of the customer data request
		                $customer_data['customform'] = $customer_data['contact.customform'];
		                unset($customer_data['contact.customform']);
	                }
                    $customer_response = $netsuite_connection->doRequest($customer_data);
                    if ($customer_response && isset($customer_response['data']['id'])) {
	                    $field = $customer_data['recordType'] == 'customer' ? "customerinternalid" : "contactinternalid";
                        update_user_meta($this->user_id, $field, $customer_response['data']['id']);
                    }
	                
                    // if we updated the contact we also want to update the company
                    if( $is_update && $customer_data['recordType'] == 'contact' && $company_data ) {
	                    $company_data['internalId'] = $this->customer_internal_id;
                    	$company_data = $this->_getCompanyData($company_data);
	                    $customer_response = $netsuite_connection->doRequest( $company_data );
                    if ($customer_response && isset($customer_response['data']['id'])) {
                        update_user_meta($this->user_id, "customerinternalid", $customer_response['data']['id']);
                    }
                }
				} else {
					$error = "Customer must have a first name, last name or company name before syncing to ns.";
					$netsuite_connection->processError( $error );
            }
        }
        }

        private function conditionalMapping(&$data, $order = NULL, $is_update = FALSE) {

            $cm_options = self::getOptions(1);


            if (!empty($cm_options)) {
                foreach ($cm_options as $mapping) {
                    if (is_null($order) && ($mapping['type'] == 3 || $mapping['type'] == 4)) {
                        continue;
                    }

                    if ($this->user_id == 0 && ($mapping['type'] == 1 || $mapping['type'] == 2)) {
                        continue;
                    }

                    if ($is_update && isset($mapping['exlcude_in_update'])) { //skip if admin wants to exlcude this update request
                        continue;
                    }
                    if (!empty($mapping['ns_field_key'])) {
                        $ns_field_key = trim($mapping['ns_field_key']);
                    } else {
                        continue;
                    }

                    $mapped_value = ConditionalMapping::ConditionalMappingOperators($mapping, $order, $this->user_id);
                    if (!empty($mapped_value)) {
                        $data[$ns_field_key] = $mapped_value;
                    }
                }
            }
            return;
        }

	    /**
	     * Allows us to restrict what data can be synced with the company
	     *
	     * @param $customer_data
	     *
	     * @return array
	     */
	    public function _getCompanyData( $customer_data ) {
		    if ( empty( $this->contact_id ) ) {
			    return $customer_data;
		    }
		
		    $customer_data['recordType'] = 'customer';
		
		    $remove_fields = [
			    'email',
			    'firstname',
			    'lastname',
			    'phone',
			    'companyname',
			    'company',
			    'isperson',
		    ];
		
		    if ( ! empty( $this->customer_internal_id ) ) {
			    $remove_fields[] = 'custentity_in8_wc_customer_id';
		    }
		
		    $remove_fields = apply_filters( 'in8_so_company_remove_fields', $remove_fields );
		
		    foreach ( $remove_fields as $field ) {
			    if ( isset( $customer_data[ $field ] ) ) {
				    // moving the data to company_data in case we need them latter
				    if( !isset($customer_data['company_data']) ){
					    $customer_data['company_data'] = [];
				    }
				    $customer_data['company_data'][$field] = $customer_data[ $field ];
				
				    unset( $customer_data[ $field ] );
			    }
		    }
		
		    return apply_filters( 'in8_so_update_company', $customer_data, $this->user_id );
	    }

        //static function for returning option tag
        public static function getOptionsTag() {
            return self::$optionsTag;
        }

        //static function for returning conditional mapping option tag
        public static function getOptionsTagCM() {
            return self::$optionsTagCM;
        }

        //static function for returning configurable options
        public static function getOptionsTagCO() {
            return self::$optionsTagCO;
        }

        // static function for returning forms and payment methods
        public static function getOptionsTagFP() {
            return self::$optionsTagFP;
        }

        /**
         * Loop through defined plugin gateways and check if they are 
         * enabled as woocommerce payment gateway
         * @return array
         */
        public static function getActivePaymentGateways() {
            $active_gateways = [];
            //get pay gateways
            $gateways = WC()->payment_gateways->payment_gateways();
            foreach ($gateways as $id => $value) {
                if (isset($gateways[$id]->enabled) && $gateways[$id]->enabled == 'yes') {
                    $active_gateways[$id] = [
                        'enabled' => true,
                        'title' => $gateways[$id]->title
                    ];
                }
            }

            return $active_gateways;
        }

        /**
         * Loop through defined plugin gateways and check if they are 
         * enabled as woocommerce payment gateway
         * @return array
         */
        private static function _getNetSuitePaymentMethods() {
            $active_gateways = [];
            //get pay gateways
            $gateways = WC()->payment_gateways->payment_gateways();

            //lopp on defined gateways
            foreach (self::$allowedGatways as $id => $name) {
                if (array_key_exists($id, $gateways)) {
                    //if gateway is enabled
                    if (isset($gateways[$id]->enabled) && $gateways[$id]->enabled == 'yes') {
                        $active_gateways[$id] = [
                            'enabled' => true,
                            'title' => $gateways[$id]->title
                        ];
                    } else {
                        $active_gateways[$id] = [
                            'enabled' => FALSE,
                            'title' => $gateways[$id]->title
                        ];
                    }
                }
            }
            return $active_gateways;
        }

        /**
         * This functions handles data saving for customers Tab
         */
        public function saveForm() {
            $request = $_REQUEST;
            //if mapping key and mapping values are not empty,
            //this code remove mapping key/value(s) if corresponsing key/value(s) is empty  
            //to eliminate empty values
            if (!empty($request['mapping_key']) && !empty($request['mapping_value'])) {
                for ($i = 0; $i < count($request['mapping_key']); $i++) {
                    if (empty($request['mapping_key'][$i])) {
                        unset($request['mapping_value'][$i]);
                        unset($request['mapping_key'][$i]);
                    }

                    if (empty($request['mapping_value'][$i])) {
                        $request['mapping_value'][$i] = "0";
                    }


                    trim($request['mapping_key'][$i]);
                    trim($request['mapping_value'][$i]);
                }
                $request['mapping_key'] = array_values($request['mapping_key']);
                $request['mapping_value'] = array_values($request['mapping_value']);
            }
            if (!empty($request)) {
                self::saveOptions($request);
            }
            die(json_encode([
                'type' => 'success',
            ]));
        }

        /**
         * This functions handles data saving for customers Tab
         */
        public function saveConfigurationalOptionsForm() {
            $request = $_REQUEST;
            //if mapping key and mapping values are not empty,
            //this code remove mapping key/value(s) if corresponsing key/value(s) is empty  
            //to eliminate empty values
            if (is_array($request['co'])) {

                $request = $request['co'];

                //if credit card payment method is not enabled, then set gateway to empty 
                if (!isset($request['enable_credit_card'])) {
                    $request['enable_credit_card'] = '';
                    $request['selected_gateway'] = '';
                }

                self::saveOptions($request, 2); //parameter two -> 0 : simple options, 1 : conditional mapping, 2: configurational options
                die(json_encode([
                    'type' => 'success',
                ]));
            }

            die(json_encode([
                'type' => 'Error',
                'message' => 'Invalid Request'
            ]));
        }

        public function getFormsAndPaymentMethodsTemplate() {

            $forms_and_payments = In8_Customer::getOptions(3);


            $return_string = "";
            if (isset($forms_and_payments['customer_forms']) && !empty($forms_and_payments['customer_forms'])) {
                $customer_forms = $forms_and_payments['customer_forms'];
                $return_string .= '<div class="well"><table class="table table-responsive table-bordered">';
                $return_string .= '<caption><h4>NetSuite Customer Forms</h4></caption>';
                $return_string .= '<thead class="warning">';
                $return_string .= '<th>Name</th>';
                $return_string .= '<th>Internal ID</th></thead><tbody>';
                foreach ($customer_forms as $customer_form) {
                    $return_string .= '<tr><td>' . $customer_form['text'] . '</td><td>' . $customer_form['internalid'] . '</td></tr>';
                }
                $return_string .= '</tbody></table></div>';
            } else {
                $return_string .= '<div class="well">Error loading custom forms from NetSuite. Please check the error logs.</div>';
            }
            
            if (isset($forms_and_payments['payment_methods']) && !empty($forms_and_payments['payment_methods'])) {
                $payment_methods = $forms_and_payments['payment_methods'];
                $return_string .= '<div class="well"><table class="table table-responsive table-bordered"><caption><h4>NetSuite Payment Methods</h4></caption><thead class="warning"><th>Name</th><th>Internal ID</th></thead><tbody>';
                foreach ($payment_methods as $payment_method) {
                    $return_string .= '<tr><td>' . $payment_method['name'] . '</td><td>' . $payment_method['internalId'] . '</td></tr>';
                }
                $return_string .= '</tbody></table></div>';
            } else {
                $return_string .= '<div class="well">Error loading payment methods from NetSuite. Please check the error logs.</div>';
            }

            return $return_string;
        }

        public function getFormsAndPaymentMethodsFromNS() {
            //GET NS fields
            $payment_methods = $customer_forms = [];
            //get a list of customer forms
            $netsuite_connection = new NetsuiteConnection('0', 'customerforms');
            $netsuite_connection_resp = $netsuite_connection->doRequest();
            //validate response
            if (isset($netsuite_connection_resp['success']) && $netsuite_connection_resp['success'] == 1 && !empty($netsuite_connection_resp['data'])) {
                $customer_forms = $netsuite_connection_resp['data'];
            }

            //get a list of payment methods
            $netsuite_connection = new NetsuiteConnection('0', 'paymentmethod');
            $netsuite_connection_resp = $netsuite_connection->doRequest();
            //validate response
            if (isset($netsuite_connection_resp['success']) && $netsuite_connection_resp['success'] == 1 && !empty($netsuite_connection_resp['data'])) {
                $payment_methods = $netsuite_connection_resp['data'];
            }

            $options = array();
            $options['customer_forms'] = $customer_forms;
            $options['payment_methods'] = $payment_methods;

            self::saveOptions($options, 3);

            $template = self::getFormsAndPaymentMethodsTemplate();

            die($template);
        }

        /**
         * @param type $options - array of all options
         * @param type $type - 0 : simple options, 1 : conditional mapping
         * @return array of all options
         */
        public static function saveOptions($options = [], $type = 0) {
            if ($type == 2) {
                $option_tag = self::$optionsTagCO;
            } elseif ($type == 1) {
                $option_tag = self::$optionsTagCM;
            } elseif ($type == 3) {
                $option_tag = self::$optionsTagFP;
            } else {
                $option_tag = self::$optionsTag;
            }

            update_option($option_tag, $options);
            return $options;
        }

        /**
         * 
         * @param type $type - 0 : simple options, 1 : conditional mapping, 2: configurational mapping
         * @return array of all options
         */
        public static function getOptions($type = 0) {
            if ($type == 2) {
                $option_tag = self::$optionsTagCO;
            } elseif ($type == 1) {
                $option_tag = self::$optionsTagCM;
            } elseif ($type == 3) {
                $option_tag = self::$optionsTagFP;
            } else {
                $option_tag = self::$optionsTag;
            }
            $options = get_option($option_tag, []);
            if (empty($options)) {
                $options = [];
            }

            return $options;
        }
	
	
	    public static function getMappings(){
		    $plugin_options = self::getOptions();
		    $plugin_settings_map        = array_change_key_case( array_combine( $plugin_options['mapping_key'], $plugin_options['mapping_value'] ) );
		    return $plugin_settings_map;
	    }

        public static function options() {
            return self::getOptions();
        }
	
	    public function create_new_address($user_id, $new_address)
	    {
		    $this->send_address_to_ns($user_id, $new_address);
	    }
	    public function update_address($user_id, $aid, $new_address)
	    {
		    $this->send_address_to_ns($user_id, $new_address, $aid);
	    }
	    public function send_address_to_ns($user_id, $new_address, $aid = false)
	    {
		    // TODO make sure user has customerid
		    //netsuite API connection
		    $netsuite_connection = new NetsuiteConnection($user_id, 'customer');
		    //customer internal ID
		    $customer_internal_id = get_user_meta($user_id, 'customerinternalid', true);
		    if( empty($customer_internal_id) ) return;
		
		    // TODO the request to netsuite per Marcel request
		    $address_type = $new_address['type'];
		    $ad2 = isset($new_address['billing_address_1']) ? 'bill' : 'ship';
		    $default_shipping = (isset($new_address[$address_type.'_is_default_address']) && $new_address[$address_type.'_is_default_address'] == 1) ? 'T' : 'F';
		
		    $customer_data = [];
		    $customer_data['recordType'] 				= "customer";
		    $customer_data['internalId'] 				= $customer_internal_id;
		    // is default address
		    $customer_data['newaddress'] = 'T';
		    $customer_data[$ad2.'attention'] 			= $new_address['address_internal_name'];
		    $customer_data[$ad2.'addressee'] 			= trim($new_address[$address_type.'first_name'] . ' ' . $new_address[$address_type.'last_name']);;
		    $customer_data[$ad2.'addrphone'] 			= $new_address[$address_type.'address_internal_name'];
		    $customer_data[$ad2.'addr1'] 				= $new_address[$address_type.'_address_1'];
		    $customer_data[$ad2.'addr2'] 				= $new_address[$address_type.'_address_2'];
		    $customer_data[$ad2.'city'] 				= $new_address[$address_type.'_city'];
		    $customer_data[$ad2.'state'] 				= $new_address[$address_type.'_state'];
		    $customer_data[$ad2.'zip'] 					= $new_address[$address_type.'_postcode'];
		    $customer_data[$ad2.'label'] 				= $new_address['address_internal_name'];
		    $customer_data['default'.$address_type] 	= $default_shipping;
		
		    update_user_meta($user_id, 'debug', [
			    'customer_data' => $customer_data
		    ]);
		    $this->conditionalMapping($customer_data, NULL, FALSE);
		
		    // save the address id for the default address on the user
		    if( $default_shipping == 'T' ){
			    $val = $aid != false ? $aid : $this->get_address_id($user_id);
			    update_user_meta($user_id, 'last_'.$address_type.'_address_id', $val);
		    }
		
		    // TODO send the info to netsuite
		    if (!empty($customer_data)) {
			    //customer create/update request
			    $customer_data = array_filter($customer_data);
			    $customer_response = $netsuite_connection->doRequest($customer_data);
			
			    // TODO get the info from netsuite and save it on wp
			    if ($customer_response && isset($customer_response['addresses'])) {
				    $ns_address_id = $this->get_address_from_ns_response($customer_response, $aid);
				
				    $this->save_internal_id_to_wc_addresses($user_id, $ns_address_id, $aid);
			    }
		    }
	    }
	    // Get internal id from netsuite address list
	    public function get_address_from_ns_response($customer_response)
	    {
		    // The last one in the list
		    $ns_address_id = '';
		    foreach($customer_response['addresses'] as $address){
			    $ns_address_id = $address['internalid'];
		    }
		    return $ns_address_id;
	    }
	    // Save the internal id in the address
	    public function save_internal_id_to_wc_addresses($user_id, $internal_id, $aid){
		    // if address id is set then use it to find the id
		    //		if aid is not set then use the last address
		    $wc_addresses = get_user_meta($user_id, '_wcmca_additional_addresses', true);
		    foreach($wc_addresses as $key => $wc_address){
			    if( $wc_address['address_id'] == $aid ){
				    $wc_addresses[$key]['internalId'] = $internal_id;
				    break;
			    }
		    }
		    // we're adding the address
		    if( $aid == false ){
			    $last = end(array_keys($wc_addresses));
			    $wc_addresses[$last]['internalId'] = $internal_id;
		    }
		
		    update_user_meta($user_id, '_wcmca_additional_addresses', $wc_addresses);
	    }
	    // If we're adding we want to get the current address id
	    public function get_address_id($user_id){
		    $wc_addresses = get_user_meta($user_id, '_wcmca_additional_addresses', true);
		    return end(array_keys($wc_addresses));
	    }
	    public function save_form_selected_address($order_id){
		    if (isset($_POST['wcmca_address_select_menu_billing'])) {
			    $val = $_POST['wcmca_address_select_menu_billing'];
			    update_post_meta($order_id, 'billing_address_id', $val);
			    if( is_numeric($val) ){
				    update_user_meta(get_current_user_id(), 'last_billing_address_id', $val);
			    }
		    }
		    if (isset($_POST['wcmca_address_select_menu_shipping'])) {
			    $val = $_POST['wcmca_address_select_menu_shipping'];
			    update_post_meta($order_id, 'shipping_address_id', $val);
			    if( is_numeric($val) ){
				    update_user_meta(get_current_user_id(), 'last_shipping_address_id', $val);
			    }
		    }
	    }
	
	    public static function testFeature($feature, $check = 'on'){
		    $options = self::getOptions( 2 );
		
		    return ! empty( $options ) && isset( $options[$feature] ) && $options[$feature] == $check;
	    }
    }

    new In8_Customer();

    