<?php

    /**
     * contains customer related functions which are common to customer and salesorder plugin
     */
    class CustomerHelperFunctions {

        /**
         * This function is used to generate customer search filter expression, 
         * a customer is searched by email by default
         * NOte : admin can enable options for search by phone or search by company or both.
         * @param type $email - customer email
         * @param type $include_company - customer conf setting for including company in search
         * @param type $company - customer company
         * @param type $include_phone - customer conf setting for including phone in search
         * @param type $phone - customer phone
         * @return string
         */
        public static function getCustomerFilterExpression($email, $include_company, $company, $include_phone, $phone, $customer_settings = []) {

            //search a customer using email in netsuite
            $filterExpression = '["email","is","' . $email . '"]';

            if ($include_company == 'on' && !empty($company)) {
            	$search_by = 'companyname';
            	if( isset($customer_settings['enable_contact_sync']) && $customer_settings['enable_contact_sync'] == 'on' ){
            		$search_by = 'company.entityid';
	            }
                $criteria = '["'.$search_by.'","is","' . $company . '"]';
                $filterExpression = $filterExpression . ',"OR",' . $criteria;
            }

            if ($include_phone == 'on' && !empty($phone)) {
                $criteria = '["phone","is","' . $phone . '"]';
                $filterExpression = $filterExpression . ',"OR",' . $criteria;
            }
            
            $filterExpression = "[" . $filterExpression . "]";
            return apply_filters('in8_customer_filter_expression', $filterExpression, $email, $include_company, $company, $include_phone, $phone, $customer_settings);
        }
        public static function getEmailEndingFilterExpression($email, $options = []) {

	        $new_email = explode('@', $email);
	        $email = '@'.$new_email[1];
	        
	        if( isset($options['search_email_domain']) && !empty($options['search_email_domain']) ){
		        $ids = array_filter(explode(',', $options['customers_entitystatus']));
	        	$entity_status = ', "AND", ["entitystatus","anyOf",["'.implode('",', $ids).'"]]';
	        }
        	
            //search a customer using email in netsuite
            $filterExpression = '[["email","contains","' . $email . '"] '.$entity_status.']';
            return $filterExpression;
        }

        /**
         * Sometimes NS customer search returns more than one record, 
         * this function extract the internalID from mutiple records
         * find the first row that has that e-mail. 
         * If doesn't find, find the first row that has that company name. 
         * If still doesn't find, find the first row that has the phone number.
         * @param type $results - array of results returned from NS customer "filterExpression" search
         * @param type $email - email to look for
         * @param type $include_company - customer conf setting
         * @param type $company - email to look for
         * @param type $include_phone - customer conf setting
         * @param type $phone - phone to look for 
         */
        public static function processCustomerSearchResult($results, $email, $include_company, $company, $include_phone, $phone, $single = true) {
            foreach ($results as $result) {
                if (strtolower($result['email']) == strtolower($email)) {
                    return $single ? $result['internalId'] : $result;
                }
            }

            if ($include_company == 'on') {
                foreach ($results as $result) {
                    if ($result['companyname'] == $company) {
                        return $single ? $result['internalId'] : $result;
                    }
                }
            }
            
            if ($include_phone == 'on') {
                foreach ($results as $result) {
                    if ($result['phone'] == $phone) {
                        return $single ? $result['internalId'] : $result;
                    }
                }
            }
            
            return 0;
        }
	    public static function processContactSearchResult($results, $email, $include_company, $company, $include_phone, $phone, $single = true) {
		    if( count($results) == 1 ){
		    	$result = $results[0];
			    if( strtolower($result['email']) == strtolower($email) ){
				    return $single ? $result['internalId'] : $result;
			    }
		    }
		    
		    foreach ($results as $key => $result) {
			    if (strtolower($result['email']) != strtolower($email)) {
				    unset($results[$key]);
			    }
		    }
		
		    if ($include_company == 'on') {
			    foreach ($results as $key => $result) {
				    $companyname1 = strtolower($result['companyname']);
				    $companyname2 = strtolower(@$result['company']['name']);
				    $company = strtolower($company);
				    if ( $companyname1 != $company
				         && ( $companyname2 ) != $company
					    // check if the contact company name matches first 4 chars
					    && substr($companyname1, 0, 4) != substr($company, 0, 4)
					    && substr($companyname2, 0, 4) != substr($company, 0, 4)
				    ) {
					    unset($results[$key]);
				    }
			    }
		    }
		
		    if ($include_phone == 'on') {
			    foreach ($results as $key => $result) {
				    if ($result['phone'] != $phone) {
					    unset($results[$key]);
				    }
			    }
		    }
		
		    if( count($results) >= 1 ){
		    	foreach($results as $result){break;}
			    return $single ? $result['internalId'] : $result;
		    }
		    return 0;
	    }

    }
    