<?php
	$in8_disable_qty_pricing = [];
	function in8sync_custom_filter_item_price( $price, $product )
	{
		
		
		$product_id = $product->get_id();
		$parent_id = 0;
		if( $product->get_type() == 'variation' ){
			$parent_id = $product->get_parent_id();
		}
		$user_id    = get_current_user_id();
		
		$mappings = In8_Customer::getOptions(2);
		$item_pricing_enabled = isset($mappings['item_pricing']) && $mappings['item_pricing'] == 'on';
		$group_pricing_enabled = isset($mappings['group_pricing']) && $mappings['group_pricing'] == 'on';
		$price_level_enabled = isset($mappings['pricing_level']) && $mappings['pricing_level'] == 'on';
		$wholesale_qty_pricing_enabled = isset($mappings['wholesale_qty_pricing']) && $mappings['wholesale_qty_pricing'] == 'on';
		$in8_qty_pricing_enabled = isset($mappings['in8_qty_pricing']) && $mappings['in8_qty_pricing'] == 'on';
		
		// Item pricing AND Group pricing are disabled
		if( !$item_pricing_enabled && !$group_pricing_enabled && !$price_level_enabled ) return $price;
		
		if ( ! empty( $user_id ) )
		{
			if ( class_exists('WC_Dynamic_Pricing_Context') )
			{
				//Cart items are discounted when loaded from session, check to see if the call to get_price is from a cart item,
				//if so, return the price on the cart item as it currently is.
				$cart_item = WC_Dynamic_Pricing_Context::instance()->get_cart_item_for_product( $product );
				
				if ( $cart_item )
				{
					
					//If no discounts applied just return the price passed to us.
					if ( ! empty( $cart_item['discounts'] ) )
					{
						return $price;
					}
				}
			}
			
			
			
			
			// Priority 1 - Individual item pricing for customers.
			if( $item_pricing_enabled ) {
				$item_pricing = get_user_meta( $user_id, 'itempricing', TRUE );
				if ( ! empty( $item_pricing ) && is_array( $item_pricing ) ) {
					// if wholesale qty pricing is on and item pricing has a level for this item use wholesale qty pricing instead
					if( checkIfItemPricingIsInWholesaleQty($product_id) ){
						return $price;
					}
					
					foreach ( $item_pricing as $itemprice ) {
						if ( $itemprice['item_id'] == $product_id && ! empty( $itemprice['price'] ) ) {
							global $in8_disable_qty_pricing;
							$in8_disable_qty_pricing[$product_id] = $product_id;
							if( !empty($parent_id) ){
								$in8_disable_qty_pricing[$parent_id] = $parent_id;
							}
							
							if ( ! empty( $itemprice['level'] ) && $itemprice['level'] != - 1 ) {
								$meta_key          = 'price_' . $itemprice['level'] . '_wholesale_price';
								$price_level_price = get_post_meta( $product_id, $meta_key, TRUE );
								if ( ! empty( $price_level_price ) ) {
									$price = number_format( (float) $price_level_price, 2, '.', '' );
								} else {
									$price = number_format( (float) $itemprice['price'], 2, '.', '' );
								}
							} else {
								$price = number_format( (float) $itemprice['price'], 2, '.', '' );
							}
							if( isset($_REQUEST['in8_debug']) ){
								echo "<pre>";print_r('item pricing: '.$price);echo "</pre>";
							}
							
							if ( class_exists( 'WWP_Wholesale_Prices' ) ) {
								add_filter( 'wwp_filter_wholesale_price_html', 'in8sync_custom_display_wholesale_price', 100, 2 );
							}
							
							return $price;
						}
					}
				}
			}
			
			
			// Priority 2 - Group pricing for customers
			if( $group_pricing_enabled ) {
				$user_group_pricing = get_user_meta( $user_id, 'group_pricing', TRUE );
				if ( ! empty( $user_group_pricing ) ) {
					$product_pricing_group = get_post_meta( $product_id, 'pricinggroup', TRUE );
					if ( ! empty( $product_pricing_group ) ) {
						foreach ( $user_group_pricing as $user_group_price ) {
							if ( $user_group_price['group'] == $product_pricing_group ) {
								$meta_key          = 'price_' . $user_group_price['level'] . '_wholesale_price';
								$price_level_price = get_post_meta( $product_id, $meta_key, TRUE );
								if ( ! empty( $price_level_price ) ) {
									global $in8_disable_qty_pricing;
									$in8_disable_qty_pricing[$product_id] = $product_id;
									if( !empty($parent_id) ){
										$in8_disable_qty_pricing[$parent_id] = $parent_id;
									}
									
									if ( class_exists( 'WWP_Wholesale_Prices' ) ) {
										add_filter( 'wwp_filter_wholesale_price_html', 'in8sync_custom_display_wholesale_price', 100, 2 );
									}
									
									return $price_level_price;
								}
							}
						}
					}
				}
			}
			
			// Priority 3 - Price level from customer record
			if( $price_level_enabled ) {
				$user_price_level = get_user_meta( $user_id, 'pricelevel', TRUE );
				if ( ! empty( $user_price_level ) ) {
					$meta_key          = 'price_' . $user_price_level . '_wholesale_price';
					$price_level_price = get_post_meta( $product_id, $meta_key, TRUE );
					if ( ! empty( $price_level_price ) ) {
						
						// wholesale quantity takes over wholesale
						if( in8_item_has_wholesale_pricing($product_id) ) return $price;
						
						if ( class_exists( 'WWP_Wholesale_Prices' ) ) {
							add_filter( 'wwp_filter_wholesale_price_html', 'in8sync_custom_display_wholesale_price', 100, 2 );
						}
						if( $_REQUEST['in8_debug'] ){
							echo "<pre>";print_r('wholesale price used: '. $price_level_price);echo "</pre>";
						}
						return $price_level_price;
					}
				}
			}
			
			// Priority 4 - Wholesale dynamic pricing
			if( $wholesale_qty_pricing_enabled && !is_checkout() && !is_cart() ) {
				$user_price_level = get_user_meta( $user_id, 'pricelevel', TRUE );
				
				// if item pricing is set get the price level from the item pricing array
				if( empty($user_price_level) ){
					$item_pricing_data = get_user_meta( $user_id, 'itempricing', TRUE );
					foreach ($item_pricing_data as $data){
						if( $data['item_id'] == $product_id ){
							$user_price_level = $data['level'];
							break;
						}
					}
				}
				
				if ( ! empty( $user_price_level ) ) {
					$meta_key          = 'in8_wholesale_qty_pricing';
					$dynamic_prices = get_post_meta( $product_id, $meta_key, TRUE );
					if ( ! empty( $dynamic_prices ) ) {
						if ( class_exists( 'WWP_Wholesale_Prices' ) ) {
							add_filter( 'wwp_filter_wholesale_price_html', 'in8sync_custom_display_wholesale_price', 100, 2 );
						}
						if( !empty($dynamic_prices[$user_price_level]) ) {
							$price_level_prices = $dynamic_prices[ $user_price_level ];
							
							
							if( $_REQUEST['in8_debug'] ){
								echo "<pre>";print_r('wholesale dynamic pricing[00] used: '. $price_level_prices[0]['price']);echo "</pre>";
							}
							return $price_level_prices[0]['price'];
						}
					}
				}
			}
		}
		
		// Priority 5 - Dynamic pricing
		if( $in8_qty_pricing_enabled && !is_checkout() && !is_cart() ) {
			$dynamic_prices = get_post_meta( $product_id, 'in8_qty_pricing', TRUE );
			if ( ! empty( $dynamic_prices ) ) {
				if ( class_exists( 'WWP_Wholesale_Prices' ) ) {
					add_filter( 'wwp_filter_wholesale_price_html', 'in8sync_custom_display_wholesale_price', 100, 2 );
				}
				if( $_REQUEST['in8_debug'] ){
					echo "<pre>";print_r('dynamic pricing[00] used: '. $dynamic_prices[0]['price']);echo "</pre>";
				}
				return $dynamic_prices[0]['price'];
			}
		}
		
		// If we've made it to here, there isn't individual item pricing for this product & customer,
		// and there isn't group pricing for this product & customer.
		// So, we just return the price
		return $price;
		
	}
	add_filter('woocommerce_product_get_price', 'in8sync_custom_filter_item_price', 1000, 2);
	add_filter('woocommerce_product_get_regular_price', 'in8sync_custom_filter_item_price', 1000, 2);
	add_filter('woocommerce_product_variation_get_price', 'in8sync_custom_filter_item_price', 1000, 2);
	add_filter('woocommerce_product_variation_get_regular_price', 'in8sync_custom_filter_item_price', 1000, 2);
	function in8sync_custom_display_wholesale_price( $wholesale_price_html , $price )
	{
		return $price;
	}
	add_filter('woocommerce_variable_price_html', 'in8sync_custom_filter_item_price_html', 1000, 2);
	function in8sync_custom_filter_item_price_html($price, $product){
		return wc_price(in8sync_custom_filter_item_price($product->get_price(), $product));
	}
	
	// quantity based pricing
	add_action( 'woocommerce_before_calculate_totals', 'in8sync_custom_filter_item_price_cart' );
	function in8sync_custom_filter_item_price_cart($cart_object){
		
		foreach ( $cart_object->get_cart() as $hash => $value ) {
			
			$item = $value['data'];
			/* @var WC_Product $item */
			$value['data']->set_price( in8sync_item_qty_based_pricing($item->get_price(), $value['quantity'], $value['product_id'], $value['variation_id']) );
		}
	}
	
	function in8sync_item_qty_based_pricing($price, $qty, $item_id, $variation_id = 0){
		if( !empty($variation_id) ) $item_id = $variation_id;
		
		$qty_pricing = in8_get_qty_based_pricing($item_id);
		
		if( !empty($qty_pricing) ) {
			foreach ( $qty_pricing as $index => $rule ) {
				if ( $rule['from'] <= $qty && ( $qty <= $rule['to'] || $rule['to'] == '' ) ) {
					
					if( isset($_REQUEST['in8_debug']) ){
						if( in8_item_has_wholesale_pricing($item_id) ){
							echo "<pre>";print_r('wholesale dynamic pricing['.$index.'] used: '. json_encode($rule));echo "</pre>";
						}else{
							echo "<pre>";print_r('dynamic pricing['.$index.'] used: '. json_encode($rule));echo "</pre>";
						}
					}
					
					return $rule['price'];
				}
			}
		}
		
		return $price;
		
	}
	// QTY PRICING FRONTEND
	// Table
	add_action( 'woocommerce_before_add_to_cart_form', 'in8_qty_based_table', 11 );
	function in8_qty_based_table(){
		global $post, $in8_disable_qty_pricing;
		
		if( isset($in8_disable_qty_pricing[$post->ID]) ) return;
		
		$mappings = In8_Customer::getOptions(2);
		$show_qty_pricing_table_enabled = isset($mappings['show_qty_pricing_table']) && $mappings['show_qty_pricing_table'] == 'on';
		if( !$show_qty_pricing_table_enabled ) return;
		
		$product = wc_get_product($post->ID);
		if( $product->get_type() == 'variable' ) {
			$variations = $product->get_available_variations();
			foreach($variations as $variation){
				in8_qty_based_pricing_table($variation['variation_id'], true);
			}
		}else{
			in8_qty_based_pricing_table($post->ID);
		}
	}
	function in8_qty_based_pricing_table($product_id, $hide = false){
		$qty_pricing = in8_get_qty_based_pricing( $product_id );
		if ( empty( $qty_pricing ) ) {
			return;
		}
		
		// check if all prices are the same
		$prices_are_same = true;
		$last_price = $qty_pricing[0]['price'];
		foreach($qty_pricing as $pricing){
			if( $last_price != $pricing['price'] ){
				$prices_are_same = false;
			}
		}
		if( $prices_are_same ) return;
		
		?>
		<style>
			.in8-qty-pricing td {
				padding: 4px;
			}
		</style>
		<div class="in8-qty-based-pricing" data-product_id="<?= $product_id ?>" style="<?= $hide ? 'display: none;' : '' ?>">
			<h5>QTY Based Pricing</h5>
			<table border="1" class="in8-qty-pricing">
				<thead>
				<tr>
					<td>Qty</td>
					<td>Price</td>
				</tr>
				</thead>
				<tbody>
				<?php foreach ( $qty_pricing as $rule ): ?>
					<tr>
						<td><?= $rule['from'] ?>-<?= $rule['to'] != '' ? $rule['to'] : '∞' ?></td>
						<td><?= wc_price( $rule['price'] ) ?></td>
					</tr>
				<?php endforeach; ?>
				</tbody>
			</table>
		</div>
		<?php
	}
	
	// Dynamic Product Pricing
	add_action('wp_enqueue_scripts', function(){
		$mappings = In8_Customer::getOptions(2);
		$use_dynamic_pricing_on_product_enabled = isset($mappings['use_dynamic_pricing_on_product']) && $mappings['use_dynamic_pricing_on_product'] == 'on';
		if( !$use_dynamic_pricing_on_product_enabled ) return;

		wp_enqueue_script('in8-item-pricing', In8_Products_Plugin_URL . "inc/assets/js/item-pricing.js");
	});
	add_action( 'woocommerce_before_add_to_cart_form', function(){
		$mappings = In8_Customer::getOptions(2);
		$use_dynamic_pricing_on_product_enabled = isset($mappings['use_dynamic_pricing_on_product']) && $mappings['use_dynamic_pricing_on_product'] == 'on';
		if( !$use_dynamic_pricing_on_product_enabled ) return;
		
		global $post, $in8_disable_qty_pricing;
		
		if( isset($in8_disable_qty_pricing[$post->ID]) ) return;
		
		$product = wc_get_product($post->ID);
		if( $product->get_type() == 'variable' ){
			/* @var WC_Product_Variable $product */
			$variations = $product->get_available_variations();
			foreach($variations as $variation){
				in8_qty_based_price_ajax($variation['variation_id']);
			}
		}else{
			in8_qty_based_price_ajax($post->ID);
		}
	}, 11 );
	function in8_qty_based_price_ajax($product_id){
		$mappings = In8_Customer::getOptions(2);
		$use_dynamic_pricing_on_product_enabled = isset($mappings['use_dynamic_pricing_on_product']) && $mappings['use_dynamic_pricing_on_product'] == 'on';
		if( !$use_dynamic_pricing_on_product_enabled ) return;
		
		$qty_pricing = in8_get_qty_based_pricing($product_id);
		
		if( empty($qty_pricing) ) return;
		foreach($qty_pricing as $key => $value){
			$qty_pricing[$key]['price_html'] = wc_price($qty_pricing[$key]['price']);
		}
		?>
		<script>
            in8_item_qty_based_pricing[<?=$product_id?>] = <?= json_encode($qty_pricing) ?>
		</script>
		<?php
	}
	
	
	function checkIfItemPricingIsInWholesaleQty($product_id){
		$mappings = In8_Customer::getOptions(2);
		
		$wholesale_qty_pricing_enabled = isset($mappings['wholesale_qty_pricing']) && $mappings['wholesale_qty_pricing'] == 'on';
		
		if( !$wholesale_qty_pricing_enabled ) return false;
		
		
		$user_id = get_current_user_id();
		// if item pricing is set get the price level from the item pricing array
		$price_level = '';
		$item_pricing_data = get_user_meta( $user_id, 'itempricing', TRUE );
		foreach ($item_pricing_data as $data){
			if( $data['item_id'] == $product_id ){
				$price_level = $data['level'];
				break;
			}
		}
		$new_qty_pricing = get_post_meta( $product_id, 'in8_wholesale_qty_pricing', TRUE );
		if( !empty($new_qty_pricing[$price_level]) ){
			return true;
		}
		
		return false;
	}
	
	// Get qty pricing array
	function in8_item_has_wholesale_pricing($product_id){
		$mappings = In8_Customer::getOptions(2);
		
		$wholesale_qty_pricing_enabled = isset($mappings['wholesale_qty_pricing']) && $mappings['wholesale_qty_pricing'] == 'on';
		
		if( !$wholesale_qty_pricing_enabled ) return false;
		
		$user_id = get_current_user_id();
		$wholesale = $user_id && $wholesale_qty_pricing_enabled;
		
		if( $wholesale ) {
			$price_level = get_user_meta( $user_id, 'pricelevel', TRUE );
			// if item pricing is set get the price level from the item pricing array
			if( empty($price_level) ){
				$item_pricing_data = get_user_meta( $user_id, 'itempricing', TRUE );
				foreach ($item_pricing_data as $data){
					if( $data['item_id'] == $product_id ){
						$price_level = $data['level'];
						break;
					}
				}
			}
			$new_qty_pricing = get_post_meta( $product_id, 'in8_wholesale_qty_pricing', TRUE );
			if( !empty($new_qty_pricing[$price_level]) ){
				$qty_pricing = $new_qty_pricing[$price_level];
			}
		}
		
		if( empty($qty_pricing) ) false;
		
		return $qty_pricing[0]['price'];
	}
	function in8_get_qty_based_pricing($product_id){
		$mappings = In8_Customer::getOptions(2);
		
		$wholesale_qty_pricing_enabled = isset($mappings['wholesale_qty_pricing']) && $mappings['wholesale_qty_pricing'] == 'on';
		$in8_qty_pricing_enabled = isset($mappings['in8_qty_pricing']) && $mappings['in8_qty_pricing'] == 'on';
		
		if( !$wholesale_qty_pricing_enabled && !$in8_qty_pricing_enabled ) return [];
		
		$user_id = get_current_user_id();
		$wholesale = $user_id && $wholesale_qty_pricing_enabled;
		
		$qty_pricing = get_post_meta( $product_id, 'in8_qty_pricing', TRUE );
		if( $wholesale ) {
			$price_level = get_user_meta( $user_id, 'pricelevel', TRUE );
			// if item pricing is set get the price level from the item pricing array
			if( empty($price_level) ){
				$item_pricing_data = get_user_meta( $user_id, 'itempricing', TRUE );
				foreach ($item_pricing_data as $data){
					if( $data['item_id'] == $product_id ){
						$price_level = $data['level'];
						break;
					}
				}
			}
			$new_qty_pricing = get_post_meta( $product_id, 'in8_wholesale_qty_pricing', TRUE );
			if( !empty($new_qty_pricing[$price_level]) ){
				$qty_pricing = $new_qty_pricing[$price_level];
			}
		}else{
			// if item is using item pricing and wholesale pricing, wholesale pricing takes over
			global $in8_disable_qty_pricing;
			if( isset($in8_disable_qty_pricing[$product_id]) ) return [];
		}
		
		
		if( empty($qty_pricing) ) return[];
		
		return $qty_pricing;
	}
	
	
	// Show message
	add_action( 'woocommerce_before_add_to_cart_form', function(){
		$mappings = In8_Customer::getOptions(2);
		if( isset($mappings['in8_pricing_message_enable']) && $mappings['in8_pricing_message_enable'] == 'on' ){
			$message = $mappings['in8_pricing_message'];
			
			global $post;
			$product = wc_get_product($post->ID);
			$style = '';
			if( $product->get_type() == 'variable' ){
				$style = 'display: none;';
			}
			echo '<p class="in8-qty-pricing-message" style="'.$style.'"><strong>'.$message.'</strong></p>';
		}
	}, 11 );
	