<?php
	
	class In8_Customers_Options {
		public function __construct() {
			
			
			$this->fix_for_download_images_from_NS();
			
			//$this->update_username_when_syncing();
			
			$this->send_notification_for_duplicate_customers();
			
			$this->access_control();
			
			
			// this will map the external id on the customer
			$this->mapExternalIdOnCustomer();
			
			require_once('item-pricing.php');
		}

		function update_username_when_syncing(  ) {
			add_action('woocommerce_api_edit_customer', function($user_id, $data){
				if( !empty($data['username']) && !empty($user_id) ) {
					global $wpdb;
					$wpdb->update($wpdb->users, ['user_login' => $data['username']], ['ID' => $user_id]);
				}
			}, 10, 2);
		}
		
		
		/**
		 * If this is enabled it will send an email notification when the search finds more than one customer or contact
		 */
		public function send_notification_for_duplicate_customers()
		{
			// DTWR - notification for multiple customers with same email
			add_action('in8_search_entities_results', function($results, $customer_email, $order_id){
				
				$internal_ids = [];
				foreach($results as $key => $result){
					// if this user is already in the list, skip it
					if( in_array($result['internalId'], $internal_ids) ){
						unset($results[$key]);
						continue;
					}
					
					$internal_ids[] = $result['internalId'];
					
					// make sure to remove contact if company is also in the results
//					if( !empty($result['company']) && isset($result['company']['internalId']) && in_array($result['company']['internalId'], $internal_ids) ){
//						unset($results[$key]);
//					}
				}
				///
				
				$count = count($results);
				if( $count > 1 ){
					
					$email_actions = IN8_Settings::getTabSettings('in8_tab_email', 'debug_emails');
					
					$recipients = isset($email_actions['duplicate_customers_recipient']) ? $email_actions['duplicate_customers_recipient'] : '';
					$subject = isset($email_actions['duplicate_customers_subject']) ? $email_actions['duplicate_customers_subject'] : '';
					if( empty($recipients) ) return;
					
					if( empty($subject) ) $subject = "Possible duplicate entity";
					
					$content = "We found {$count} customers/contacts when searching with {$customer_email}!";
					$content .= "<br/>";
					$content .= "<ul>";
					foreach($results as $key => $result){
						$link = 'https://system.netsuite.com/app/common/entity/custjob.nl?id=';
						if( !empty($result['company']) ){
							$link = 'https://system.netsuite.com/app/common/entity/contact.nl?id=';
						}
						$link .= $result['internalId'];
						$content .= "<li><a href='{$link}'>{$link}</a></li>";
					}
					$content .= "</ul>";
					
					wp_mail($recipients,$subject,$content, ['Content-Type: text/html; charset=UTF-8']);
				}
			}, 10, 3);
		}
		
		
		/**
		 * This feature allows the client to control on NS who has access to the website and who doesn't
		*/
		public function access_control(){
			add_action('init', function(){
				$co_options = In8_Customer::getOptions( 2 );
				if ( empty( $co_options['access_control'] ) ) {
					return;
				}
				
				add_filter( 'wp_authenticate_user', function($user) use($co_options) {
					if ( !is_wp_error( $user ) ) {
						
						if( isset($user->caps['administrator']) ) return $user;
						
						$gaccess = get_user_meta( $user->ID, 'give_access', true );
						if ( $gaccess != 'T' ) {
							$access_control_message = empty($co_options['access_control_message']) ? 'Sorry! Your account does not have access.' : $co_options['access_control_message'];
							
							return new WP_Error( 'in8_login_error', __( '<strong>ERROR:</strong>' . $access_control_message ) );
						}
					}
					return $user;
				}, 100000 );
			});
		}
		
		
		/**
		 *    This function will check if the use of external id is on
		 *    If it is on it will grab the prefix that was saved or default to wc_
		 *    It will map the externalid field to the prefix and order id, eg: wc_123
		 */
		public function mapExternalIdOnCustomer() {
			add_filter( 'in8_so_customer_after_prepared_data', function ( $default, $order ) {
				/* @var WC_Order $order */
				// generic item
				$co_options = In8_Customer::getOptions( 2 );
				if ( empty( $co_options['use_external_id'] ) ) {
					return $default;
				}
				
				if( $default['recordType'] == 'customer' ){
					$prefix = empty( $co_options['prefix_customer_externalid'] ) ? 'wc_' : $co_options['prefix_customer_externalid'];
				}
				
				if( !empty($prefix) ){
					if ( ! empty( $order->get_customer_id() ) ) {
						$externalId = $prefix . $order->get_customer_id();
					} else {
						$externalId = $prefix . $order->get_billing_email();
					}
					$default['externalid'] = $externalId;
				}
				
				return $default;
			}, 10, 2 );
			add_filter( 'in8sync_before_contact_sync', function ( $default, $customer_internal_id, $user_id, $order_id ) {
				/* @var WC_Order $order */
				// generic item
				
				if( empty($order_id) ) return $default;
				
				$order = wc_get_order($order_id);
				
				$co_options = In8_Customer::getOptions( 2 );
				if ( empty( $co_options['use_external_id'] ) ) {
					return $default;
				}
				
				$prefix = empty( $co_options['prefix_contact_externalid'] ) ? 'wc_' : $co_options['prefix_contact_externalid'];
				
				if( !empty($prefix) ){
					if ( ! empty( $order->get_customer_id() ) ) {
						$externalId = $prefix . $order->get_customer_id();
					} else {
						$externalId = $prefix . $order->get_billing_email();
					}
					$default['externalid'] = $externalId;
				}
				
				return $default;
			}, 10, 4 );
		}
		
		
		/**
		 * This script will fix the issue with Woocommerce not downloading the image on the new versions
		 */
		public function fix_for_download_images_from_NS(){
			add_action('plugins_loaded', function(){
				if ( !class_exists( 'WooCommerce' ) ) return;
				
				global $woocommerce;
				if ( !version_compare( $woocommerce->version, '3.5', ">=" ) ) return;
				
				
				define('ALLOW_UNFILTERED_UPLOADS', true);
				add_filter('wp_handle_sideload_prefilter', function($data){
					if( $data['name'] == 'media.nl' ){
						if( function_exists('finfo_open') ){
							$finfo = finfo_open(FILEINFO_MIME_TYPE);
							$content_type = finfo_file($finfo, $data['tmp_name']);
							if ( !empty($content_type) && strstr( $content_type, 'image/' ) ) {
								$data['name'] = 'image.' . str_replace( 'image/', '', $content_type );
							}
						}else{
							$data['name'] = 'image.jpg';
						}
					}
					return $data;
				});
			}, 100);
		}
		
	}
	
	new In8_Customers_Options();